@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf8 -*-
 # ============================================================================
-#  Copyright (c) 2013-2020 nexB Inc. http://www.nexb.com/ - All rights reserved.
+#  Copyright (c) nexB Inc. http://www.nexb.com/ - All rights reserved.
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
@@ -13,31 +13,22 @@
 #  limitations under the License.
 # ============================================================================
 
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
-from collections import Counter
-from collections import OrderedDict
 import io
 import json
+from collections import Counter, OrderedDict
+from itertools import zip_longest
 
 import attr
+import itertools
+import openpyxl
 
 from attributecode import CRITICAL
 from attributecode import Error
 from attributecode import saneyaml
 from attributecode.util import csv
-from attributecode.util import python2
 from attributecode.util import replace_tab_with_spaces
 
 
-if python2:  # pragma: nocover
-    from itertools import izip_longest as zip_longest  # NOQA
-else:  # pragma: nocover
-    from itertools import zip_longest  # NOQA
-
-
 def transform_csv_to_csv(location, output, transformer):
     """
     Read a CSV file at `location` and write a new CSV file at `output`. Apply
@@ -51,18 +42,18 @@ def transform_csv_to_csv(location, output, transformer):
 
     errors = []
     data = iter(rows)
-    field_names = next(rows)
-
+    names = next(rows)
+    field_names = strip_trailing_fields_csv(names)
     dupes = check_duplicate_fields(field_names)
 
     if dupes:
         msg = u'Duplicated field name: %(name)s'
         for name in dupes:
             errors.append(Error(CRITICAL, msg % locals()))
-        return field_names, [], errors
+        return errors
 
     # Convert to dicts
-    new_data = [OrderedDict(zip_longest(field_names, item)) for item in data]
+    new_data = [dict(zip_longest(field_names, item)) for item in data]
 
     field_names, updated_data, errors = transform_data(new_data, transformer)
 
@@ -82,10 +73,11 @@ def transform_json_to_json(location, output, transformer):
     if not transformer:
         raise ValueError('Cannot transform without Transformer')
 
-    data = read_json(location)
-    new_data = normalize_dict_data(data)
+    items = read_json(location)
+    data = normalize_dict_data(items)
+    new_data = strip_trailing_fields_json(data)
 
-    field_names, updated_data, errors = transform_data(new_data, transformer)
+    _field_names, updated_data, errors = transform_data(new_data, transformer)
 
     if errors:
         return errors
@@ -94,6 +86,55 @@ def transform_json_to_json(location, output, transformer):
         return []
 
 
+def transform_excel_to_excel(location, output, transformer):
+    """
+    Read a XLSX file at `location` and write a new Excel file at `output`. Apply
+    transformations using the `transformer` Transformer.
+    Return a list of Error objects.
+    """
+    if not transformer:
+        raise ValueError('Cannot transform without Transformer')
+
+    dupes, new_data = read_excel(location)
+    errors = []
+    if dupes:
+        msg = u'Duplicated field name: %(name)s'
+        for name in dupes:
+            errors.append(Error(CRITICAL, msg % locals()))
+        return errors
+
+    _field_names, updated_data, errors = transform_data(new_data, transformer)
+    if errors:
+        return errors
+    else:
+        write_excel(output, updated_data)
+        return []
+
+
+def strip_trailing_fields_csv(names):
+    """
+    Strip trailing spaces for field names #456
+    """
+    field_names = []
+    for name in names:
+        field_names.append(name.strip())
+    return field_names
+
+
+def strip_trailing_fields_json(items):
+    """
+    Strip trailing spaces for field name #456
+    """
+    data = []
+    for item in items:
+        od = {}
+        for field in item:
+            stripped_field_name = field.strip()
+            od[stripped_field_name] = item[field]
+        data.append(od)
+    return data
+
+
 def normalize_dict_data(data):
     """
     Check if the input data from scancode-toolkit and normalize to a normal
@@ -103,7 +144,7 @@ def normalize_dict_data(data):
     try:
         # Check if this is a JSON output from scancode-toolkit
         if(data["headers"][0]["tool_name"] == "scancode-toolkit"):
-            #only takes data inside "files"
+            # only takes data inside "files"
             new_data = data["files"]
     except:
         new_data = data
@@ -241,7 +282,7 @@ class Transformer(object):
         Load and return a Transformer instance from a YAML configuration file at
         `location`.
         """
-        with io.open(location, encoding='utf-8') as conf:
+        with io.open(location, encoding='utf-8', errors='replace') as conf:
             data = saneyaml.load(replace_tab_with_spaces(conf.read()))
         return cls(
             field_renamings=data.get('field_renamings', {}),
@@ -282,7 +323,7 @@ class Transformer(object):
 
         renamed_list = []
         for row in data:
-            renamed = OrderedDict()
+            renamed = {}
             for key in row:
                 matched = False
                 for renamed_key in renamings:
@@ -303,29 +344,28 @@ class Transformer(object):
             return field_names
         return [c.strip().lower() for c in field_names]
     """
+
     def filter_fields(self, data):
         """
         Yield transformed dicts from a `data` list of dicts keeping only
         fields with a name in the `field_filters`of this Transformer.
         Return the data unchanged if no `field_filters` exists.
         """
-        #field_filters = set(self.clean_fields(self.field_filters))
+        # field_filters = set(self.clean_fields(self.field_filters))
         field_filters = set(self.field_filters)
         for entry in data:
-            items = ((k, v) for k, v in entry.items() if k in field_filters)
-            yield OrderedDict(items)
+            yield {k: v for k, v in entry.items() if k in field_filters}
 
     def filter_excluded(self, data):
         """
-        Yield transformed dicts from a `data` list of dicts excluding 
+        Yield transformed dicts from a `data` list of dicts excluding
         fields with names in the `exclude_fields`of this Transformer.
         Return the data unchanged if no `exclude_fields` exists.
         """
-        #exclude_fields = set(self.clean_fields(self.exclude_fields))
+        # exclude_fields = set(self.clean_fields(self.exclude_fields))
         exclude_fields = set(self.exclude_fields)
         for entry in data:
-            items = ((k, v) for k, v in entry.items() if k not in exclude_fields)
-            yield OrderedDict(items)
+            yield {k: v for k, v in entry.items() if k not in exclude_fields}
 
 
 def check_duplicate_fields(field_names):
@@ -352,8 +392,7 @@ def read_json(location):
     Yield rows (as a list of values) from a CSV file at `location`.
     """
     with io.open(location, encoding='utf-8', errors='replace') as jsonfile:
-        data = json.load(jsonfile, object_pairs_hook=OrderedDict)
-        return data
+        return json.load(jsonfile)
 
 
 def write_csv(location, data, field_names):  # NOQA
@@ -361,7 +400,7 @@ def write_csv(location, data, field_names):  # NOQA
     Write a CSV file at `location` the `data` list of ordered dicts using the
     `field_names`.
     """
-    with io.open(location, 'w', encoding='utf-8', newline='\n') as csvfile:
+    with io.open(location, 'w', encoding='utf-8', newline='\n', errors='replace') as csvfile:
         writer = csv.DictWriter(csvfile, fieldnames=field_names)
         writer.writeheader()
         writer.writerows(data)
@@ -373,3 +412,54 @@ def write_json(location, data):
     """
     with open(location, 'w') as jsonfile:
         json.dump(data, jsonfile, indent=3)
+
+def read_excel(location):
+    """
+    Read XLSX at `location`, return a list of ordered dictionaries, one
+    for each row.
+    """
+    results = []
+    errors = []
+    sheet_obj = openpyxl.load_workbook(location).active
+    max_col = sheet_obj.max_column
+
+    index = 1
+    col_keys = []
+    mapping_dict = {}
+    while index <= max_col:
+        value = sheet_obj.cell(row=1, column=index).value
+        if value in col_keys:
+            msg = 'Duplicated column name, ' + str(value) + ', detected.' 
+            errors.append(Error(CRITICAL, msg))
+            return errors, results
+        if value in mapping_dict:
+            value = mapping_dict[value]
+        col_keys.append(value)
+        index = index + 1
+
+    for row in sheet_obj.iter_rows(min_row=2, values_only=True):
+        row_dict = OrderedDict()
+        index = 0
+        while index < max_col:
+            value = row[index]
+            if value:
+                row_dict[col_keys[index]] = value
+            else:
+                row_dict[col_keys[index]] = ''
+            index = index + 1
+        results.append(row_dict)
+    return errors, results
+
+
+def write_excel(location, data):
+    wb = openpyxl.Workbook()
+    ws = wb.active
+
+    # Get the header
+    headers = list(data[0].keys())
+    ws.append(headers)
+
+    for elements in data:
+        ws.append([elements.get(h) for h in headers])
+
+    wb.save(location)
