@@ -2,7 +2,7 @@
 # -*- coding: utf8 -*-
 
 # ============================================================================
-#  Copyright (c) 2014-2017 nexB Inc. http://www.nexb.com/ - All rights reserved.
+#  Copyright (c) nexB Inc. http://www.nexb.com/ - All rights reserved.
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
@@ -13,11 +13,6 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 # ============================================================================
-
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import unittest
 
 import mock
@@ -42,18 +37,14 @@ class ApiTest(unittest.TestCase):
     @mock.patch.object(api, 'request_license_data')
     def test_api_get_license_details_from_api(self, request_license_data):
         license_data = {
-            'name': 'Apache License 2.0',
+            'short_name': 'Apache 2.0',
             'full_text': 'Apache License Version 2.0 ...',
             'key': 'apache-2.0',
         }
         errors = []
         request_license_data.return_value = license_data, errors
 
-        expected = (
-            'Apache License 2.0',
-            'apache-2.0',
-            'Apache License Version 2.0 ...',
-            [])
+        expected = ({'short_name': 'Apache 2.0', 'full_text': 'Apache License Version 2.0 ...', 'key': 'apache-2.0'}, [])
         result = api.get_license_details_from_api(
             api_url='api_url', api_key='api_key', license_key='license_key')
         assert expected == result
