@@ -2,7 +2,7 @@
 # -*- coding: utf8 -*-
 
 # ============================================================================
-#  Copyright (c) 2014-2020 nexB Inc. http://www.nexb.com/ - All rights reserved.
+#  Copyright (c) nexB Inc. http://www.nexb.com/ - All rights reserved.
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
@@ -14,11 +14,6 @@
 #  limitations under the License.
 # ============================================================================
 
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
-from collections import OrderedDict
 import io
 import json
 import os
@@ -35,7 +30,7 @@ from attributecode import INFO
 from attributecode import WARNING
 from attributecode import Error
 from attributecode import model
-from attributecode.util import add_unc
+from attributecode.util import add_unc, norm, on_windows
 from attributecode.util import load_csv
 from attributecode.util import to_posix
 from attributecode.util import replace_tab_with_spaces
@@ -45,13 +40,6 @@ from testing_utils import get_temp_dir
 from testing_utils import get_temp_file
 from testing_utils import get_test_loc
 
-try:
-    # Python 2
-    unicode  # NOQA
-except NameError:  # pragma: nocover
-    # Python 3
-    unicode = str  # NOQA
-
 
 def check_csv(expected, result, regen=False, fix_cell_linesep=False):
     """
@@ -78,7 +66,7 @@ def fix_crlf(items):
     This is fixing this until we find can why
     """
     for key, value in items:
-        if isinstance(value, unicode) and '\r\n' in value:
+        if isinstance(value, str) and '\r\n' in value:
             value = value.replace('\r\n', '\n')
         yield key, value
 
@@ -88,9 +76,9 @@ def check_json(expected, result):
     Assert that the contents of two JSON files are equal.
     """
     with open(expected) as e:
-        expected = json.load(e, object_pairs_hook=OrderedDict)
+        expected = json.load(e, object_pairs_hook=dict)
     with open(result) as r:
-        result = json.load(r, object_pairs_hook=OrderedDict)
+        result = json.load(r, object_pairs_hook=dict)
     assert expected == result
 
 
@@ -105,11 +93,12 @@ def get_unicode_content(location):
     """
     Read file at location and return a unicode string.
     """
-    with io.open(location, encoding='utf-8') as doc:
+    with io.open(location, encoding='utf-8', errors='replace') as doc:
         return doc.read()
 
 
 class FieldTest(unittest.TestCase):
+
     def test_Field_init(self):
         model.Field()
         model.StringField()
@@ -237,7 +226,7 @@ class FieldTest(unittest.TestCase):
     def test_PathField_contains_dict_after_validate(self):
         value = 'string'
         field_class = model.PathField
-        expected = OrderedDict([('string', None)])
+        expected = dict([('string', None)])
         expected_errors = [
             Error(ERROR, 'Field s: Unable to verify path: string: No base directory provided')
                           ]
@@ -254,6 +243,7 @@ class FieldTest(unittest.TestCase):
 
 class YamlParseTest(unittest.TestCase):
     maxDiff = None
+
     def test_saneyaml_load_can_parse_simple_fields(self):
         test = get_test_content('test_model/parse/basic.about')
         result = saneyaml.load(test)
@@ -388,6 +378,7 @@ test with no colon
         except Exception:
             pass
 
+
 class AboutTest(unittest.TestCase):
 
     def test_About_load_ignores_original_field_order_and_uses_standard_predefined_order(self):
@@ -411,7 +402,6 @@ class AboutTest(unittest.TestCase):
             Error(WARNING, 'Field Name is a duplicate. Original value: "old" replaced with: "new"')
         ]
 
-
         result = a.errors
         assert sorted(expected) == sorted(result)
 
@@ -440,9 +430,9 @@ class AboutTest(unittest.TestCase):
             'about_resource'])
 
         expected_errors = [
-            Error(INFO, 'Field date is a custom field.'),
-            Error(INFO, 'Field license_spdx is a custom field.'),
-            Error(INFO, 'Field license_text_file is a custom field.')]
+            Error(INFO, 'Custom Field: date'),
+            Error(INFO, 'Custom Field: license_spdx'),
+            Error(INFO, 'Custom Field: license_text_file')]
 
         errors = about.hydrate(fields)
 
@@ -539,9 +529,10 @@ class AboutTest(unittest.TestCase):
         test_file = get_test_loc('test_model/parse/illegal_custom_field.about')
         a = model.About(test_file)
         expected_errors = [
-            Error(INFO, 'Field hydrate is a custom field.'),
+            Error(INFO, 'Custom Field: hydrate'),
             Error(CRITICAL, "Internal error with custom field: 'hydrate': 'illegal name'.")
         ]
+
         assert expected_errors == a.errors
         assert not hasattr(getattr(a, 'hydrate'), 'value')
         field = list(a.custom_fields.values())[0]
@@ -591,7 +582,7 @@ this software and releases the component to Public Domain.
         test_file = get_test_loc('test_model/parse/non_ascii_field_name_value.about')
         a = model.About(test_file)
         expected = [
-            Error(CRITICAL, "Field name: 'mat\xedas' contains illegal name characters: 0 to 9, a to z, A to Z and _. (or empty spaces)")
+            Error(ERROR, "Field name: ['mat\xedas'] contains illegal name characters (or empty spaces) and is ignored.")
         ]
         assert expected == a.errors
 
@@ -688,7 +679,7 @@ this software and releases the component to Public Domain.
     def test_comma_in_license(self):
         test_file = get_test_loc('test_model/special_char/about.ABOUT')
         a = model.About(test_file)
-        expected = Error(ERROR,  "The following character(s) cannot be in the license_key: [',']")
+        expected = Error(ERROR, "The following character(s) cannot be in the license_key: [',']")
         assert a.errors[0] == expected
 
     def test_load_dict_issue_433(self):
@@ -700,8 +691,8 @@ this software and releases the component to Public Domain.
             'license_expression': 'license1 AND license2',
             'notice_file': 'package1.zip.NOTICE',
             'licenses': [
-                {'key': 'license1', 'name': 'License1', 'file': 'license1.LICENSE'},
-                {'key': 'license2', 'name': 'License2', 'file': 'license2.LICENSE'},
+                {'key': 'license1', 'name': 'License1', 'file': 'license1.LICENSE', 'url': 'some_url', 'spdx_license_key': 'key'},
+                {'key': 'license2', 'name': 'License2', 'file': 'license2.LICENSE', 'url': 'some_url', 'spdx_license_key': 'key'},
             ],
         }
         about = model.About()
@@ -717,13 +708,20 @@ licenses:
   - key: license1
     name: License1
     file: license1.LICENSE
+    url: some_url
+    spdx_license_key: key
   - key: license2
     name: License2
     file: license2.LICENSE
+    url: some_url
+    spdx_license_key: key
 '''
-        assert about.dumps() == expected
+        lic_dict = {u'license1': [u'License1', u'license1.LICENSE',u'', u'some_url', 'key'], u'license2' : [u'License2', u'license2.LICENSE', u'', u'some_url', 'key']}
+        assert about.dumps(lic_dict) == expected
+
 
 class SerializationTest(unittest.TestCase):
+
     def test_About_dumps(self):
         test_file = get_test_loc('test_model/dumps/about.ABOUT')
         a = model.About(test_file)
@@ -746,6 +744,7 @@ vcs_tool: git
 vcs_repository: https://github.com/dejacode/about-code-tool.git
 licenses:
   - key: apache-2.0
+    name: Apache 2.0
     file: apache-2.0.LICENSE
 '''
         result = a.dumps()
@@ -755,8 +754,8 @@ licenses:
         test_file = get_test_loc('test_model/parse/complete2/about.ABOUT')
         a = model.About(test_file)
         expected_error = [
-            Error(INFO, 'Field custom1 is a custom field.'),
-            Error(INFO, 'Field custom2 is a custom field.'),
+            Error(INFO, 'Custom Field: custom1'),
+            Error(INFO, 'Custom Field: custom2'),
             Error(INFO, 'Field custom2 is present but empty.')
         ]
         assert sorted(expected_error) == sorted(a.errors)
@@ -795,8 +794,8 @@ modified: yes
         test_file = get_test_loc('test_model/parse/complete2/about.ABOUT')
         a = model.About(test_file)
         expected_error = [
-            Error(INFO, 'Field custom1 is a custom field.'),
-            Error(INFO, 'Field custom2 is a custom field.'),
+            Error(INFO, 'Custom Field: custom1'),
+            Error(INFO, 'Custom Field: custom2'),
             Error(INFO, 'Field custom2 is present but empty.')
         ]
         assert sorted(expected_error) == sorted(a.errors)
@@ -844,24 +843,23 @@ custom1: |
         file_path = posixpath.join(posixpath.dirname(test_file), 'nose-selecttests-0.3.zip')
         err_msg = 'Field about_resource: Path %s not found' % file_path
         errors = [
-            Error(INFO, 'Field dje_license is a custom field.'),
-            Error(INFO, 'Field license_text_file is a custom field.'),
-            Error(INFO, 'Field scm_tool is a custom field.'),
-            Error(INFO, 'Field scm_repository is a custom field.'),
-            Error(INFO, 'Field test is a custom field.'),
+            Error(INFO, 'Custom Field: dje_license'),
+            Error(INFO, 'Custom Field: license_text_file'),
+            Error(INFO, 'Custom Field: scm_tool'),
+            Error(INFO, 'Custom Field: scm_repository'),
+            Error(INFO, 'Custom Field: test'),
             Error(INFO, err_msg)]
 
         assert errors == a.errors
         assert 'Copyright (c) 2012, Domen Kožar' == a.copyright.value
 
-    def test_load_has_errors_for_non_unicode(self):
+    def test_load_non_unicode(self):
         test_file = get_test_loc('test_model/unicode/not-unicode.ABOUT')
         a = model.About()
         a.load(test_file)
         err = a.errors[0]
         assert CRITICAL == err.severity
         assert 'Cannot load invalid ABOUT file' in err.message
-        assert 'UnicodeDecodeError' in err.message
 
     def test_as_dict_load_dict_ignores_empties(self):
         test = {
@@ -877,7 +875,7 @@ custom1: |
 
         expected = {
             'about_file_path': None,
-            'about_resource': OrderedDict([('.', None)]),
+            'about_resource': dict([('.', None)]),
             'copyright': 'Copyright (c) 2013-2014 nexB Inc.',
             'custom1': 'some custom',
             'description': 'AboutCode is a tool\nfor files.',
@@ -913,7 +911,7 @@ custom1: |
 
         expected = {
             'about_file_path': None,
-            'about_resource': OrderedDict([('.', None)]),
+            'about_resource': dict([('.', None)]),
             'attribute': 'yes',
             'author': 'Jillian Daguil, Chin Yeung Li, Philippe Ombredanne, Thomas Druez',
             'copyright': 'Copyright (c) 2013-2014 nexB Inc.',
@@ -949,7 +947,7 @@ custom1: |
 
         about_data.update({
             'about_file_path': None,
-            'about_resource': OrderedDict([('.', None)]),
+            'about_resource': dict([('.', None)]),
         })
         assert about_data == about.as_dict()
 
@@ -1022,6 +1020,7 @@ This component is released to the public domain by the author.
 '''
         assert notice_context == expected_notice
 
+
 class CollectorTest(unittest.TestCase):
 
     def test_collect_inventory_return_errors(self):
@@ -1033,8 +1032,7 @@ class CollectorTest(unittest.TestCase):
         err_msg1 = 'non-supported_date_format.ABOUT: Field about_resource: Path %s not found' % file_path1
         err_msg2 = 'supported_date_format.ABOUT: Field about_resource: Path %s not found' % file_path2
         expected_errors = [
-            Error(INFO, 'non-supported_date_format.ABOUT: Field date is a custom field.'),
-            Error(INFO, 'supported_date_format.ABOUT: Field date is a custom field.'),
+            Error(INFO, "Field ['date'] is a custom field."),
             Error(INFO, err_msg1),
             Error(INFO, err_msg2)]
         assert sorted(expected_errors) == sorted(errors)
@@ -1071,7 +1069,7 @@ class CollectorTest(unittest.TestCase):
         result = [a.about_file_path for a in abouts]
         assert expected == result
 
-    def test_collect_inventory_return_no_warnings_and_model_can_uuse_relative_paths(self):
+    def test_collect_inventory_return_no_warnings_and_model_can_use_relative_paths(self):
         test_loc = get_test_loc('test_model/rel/allAboutInOneDir')
         errors, _abouts = model.collect_inventory(test_loc)
         expected_errors = []
@@ -1107,18 +1105,17 @@ class CollectorTest(unittest.TestCase):
     def test_collect_inventory_always_collects_custom_fieldsg(self):
         test_loc = get_test_loc('test_model/inventory/custom_fields.ABOUT')
         errors, abouts = model.collect_inventory(test_loc)
-        expected_msg1 = 'Field resource is a custom field'
-        assert len(errors) == 2
-        assert expected_msg1 in errors[0].message
-        # The not supported 'resource' value is collected
+        expected_msg = "Field ['resource', 'custom_mapping'] is a custom field."
+        assert len(errors) == 1
+        assert expected_msg in errors[0].message
+        # The value of the custom field: 'resource' is collected
         assert abouts[0].resource.value
 
     def test_collect_inventory_does_not_raise_error_and_maintains_order_on_custom_fields(self):
         test_loc = get_test_loc('test_model/inventory/custom_fields2.ABOUT')
         errors, abouts = model.collect_inventory(test_loc)
         expected_errors = [
-            Error(INFO, 'inventory/custom_fields2.ABOUT: Field resource is a custom field.'),
-            Error(INFO, 'inventory/custom_fields2.ABOUT: Field custom_mapping is a custom field.')
+            Error(INFO, "Field ['resource', 'custom_mapping'] is a custom field.")
         ]
         assert expected_errors == errors
         expected = [u'about_resource: .\nname: test\nresource: .\ncustom_mapping: test\n']
@@ -1209,13 +1206,65 @@ class CollectorTest(unittest.TestCase):
         location = get_test_loc('test_model/crlf/about.ABOUT')
         result = get_temp_file()
         errors, abouts = model.collect_inventory(location)
-        errors2 = model.write_output(abouts, result, format='csv')
-        errors.extend(errors2)
+        model.write_output(abouts, result, format='csv')
         assert all(e.severity == INFO for e in errors)
 
         expected = get_test_loc('test_model/crlf/expected.csv')
         check_csv(expected, result, fix_cell_linesep=True, regen=False)
 
+    def test_copy_redist_src_no_structure(self):
+        test_loc = get_test_loc('test_model/redistribution/')
+        copy_list = [get_test_loc('test_model/redistribution/this.c'), get_test_loc('test_model/redistribution/test/subdir')]
+        output = get_temp_dir()
+
+        expected_file = ['this.c', 'subdir']
+
+        with_structure = False
+        err = model.copy_redist_src(copy_list, test_loc, output, with_structure)
+
+        assert err == []
+
+        from os import listdir
+        copied_files = listdir(output)
+        assert len(expected_file) == len(copied_files)
+        assert err == []
+        for file in expected_file:
+            assert file in copied_files
+
+    def test_copy_redist_src_with_structure(self):
+        test_loc = get_test_loc('test_model/redistribution/')
+        copy_list = [get_test_loc('test_model/redistribution/this.c'), get_test_loc('test_model/redistribution/test/subdir')]
+        output = get_temp_dir()
+
+        expected_file = ['this.c', 'test']
+
+        with_structure = True
+        err = model.copy_redist_src(copy_list, test_loc, output, with_structure)
+
+        assert err == []
+
+        from os import listdir
+        copied_files = listdir(output)
+        assert len(expected_file) == len(copied_files)
+        assert err == []
+        for file in expected_file:
+            assert file in copied_files
+
+    def test_get_copy_list(self):
+        location = get_test_loc('test_model/redistribution/')
+        result = get_temp_file()
+        errors, abouts = model.collect_inventory(location)
+        copy_list, err = model.get_copy_list(abouts, location)
+        assert err == []
+        expected = [os.path.join(location, 'this.c'), os.path.join(location, 'test/subdir')]
+        if on_windows:
+            norm_list = []
+            for c in copy_list:
+                norm_list.append(norm(c))
+            assert norm_list == expected
+        else:
+            assert copy_list == expected
+
 
 class FetchLicenseTest(unittest.TestCase):
 
@@ -1226,16 +1275,32 @@ class FetchLicenseTest(unittest.TestCase):
 
     @mock.patch('attributecode.util.have_network_connection')
     @mock.patch('attributecode.model.valid_api_url')
-    def test_pre_process_and_fetch_license_dict(self, have_network_connection, valid_api_url):
+    def test_pre_process_and_fetch_license_dict_dje(self, have_network_connection, valid_api_url):
         have_network_connection.return_value = True
-
         valid_api_url.return_value = False
         error_msg = (
             'Network problem. Please check your Internet connection. '
             'License generation is skipped.')
         expected = ({}, [Error(ERROR, error_msg)])
-        assert model.pre_process_and_fetch_license_dict([], '', '') == expected
+        assert model.pre_process_and_fetch_license_dict([]) == expected
 
         valid_api_url.return_value = True
         expected = ({}, [])
-        assert model.pre_process_and_fetch_license_dict([], '', '') == expected
+        assert model.pre_process_and_fetch_license_dict([]) == expected
+
+    @mock.patch('attributecode.util.have_network_connection')
+    @mock.patch('attributecode.model.valid_api_url')
+    def test_pre_process_and_fetch_license_dict_licensedb(self, have_network_connection, valid_api_url):
+        have_network_connection.return_value = False
+        valid_api_url.return_value = False
+        error_msg = (
+            'Network problem. Please check your Internet connection. '
+            'License generation is skipped.')
+        expected = ({}, [Error(ERROR, error_msg)])
+        assert model.pre_process_and_fetch_license_dict([]) == expected
+
+        have_network_connection.return_value = True
+        valid_api_url.return_value = True
+        expected = ({}, [])
+
+        assert model.pre_process_and_fetch_license_dict([]) == expected
\ No newline at end of file
